/*******************************************************************************
* NXP Semiconductors
* (c) Copyright 2016 NXP Semiconductors
* ALL RIGHTS RESERVED.
********************************************************************************
Services performed by NXP in this matter are performed AS IS and without
any warranty. CUSTOMER retains the final decision relative to the total design
and functionality of the end product. NXP neither guarantees nor will be
held liable by CUSTOMER for the success of this project.
NXP DISCLAIMS ALL WARRANTIES, EXPRESSED, IMPLIED OR STATUTORY INCLUDING,
BUT NOT LIMITED TO, IMPLIED WARRANTY OF MERCHANTABILITY OR FITNESS FOR
A PARTICULAR PURPOSE ON ANY HARDWARE, SOFTWARE ORE ADVISE SUPPLIED 
TO THE PROJECT BY NXP, AND OR NAY PRODUCT RESULTING FROM NXP SERVICES. 
IN NO EVENT SHALL NXP BE LIABLE FOR INCIDENTAL OR CONSEQUENTIAL DAMAGES ARISING
OUT OF THIS AGREEMENT. 
CUSTOMER agrees to hold NXP harmless against any and all claims demands 
or actions by anyone on account of any damage, or injury, whether commercial,
contractual, or tortuous, rising directly or indirectly as a result 
of the advise or assistance supplied CUSTOMER in connection with product, 
services or goods supplied under this Agreement.
********************************************************************************
* File:             main.c
* Owner:            b21190(Vlna Peter)
* Version:          1.0
* Date:             Oct-22-2014
* Classification:   General Business Information
* Brief:            DRUN mode with max core frequency(200MHz) generated from PPL0
********************************************************************************
********************************************************************************
* Detailed Description:
*
* 
* ------------------------------------------------------------------------------
* Test HW:  MPC57xx
* Maskset:  1N65H
* Target :  internal_FLASH
* Fsys:     200 MHz PLL with 40 MHz crystal reference
*           
*
********************************************************************************
Revision History:
1.0     Apr-04-2016     b21190(Vlna Peter)  Initial Version
1.1     Oct-11-2017     b21190(Vlna Peter)  Added stop mode
*******************************************************************************/

/*******************************************************************************
* Includes                                                                     
*******************************************************************************/
#include "MPC5744P.h"

/*******************************************************************************
* Constants and macros
*******************************************************************************/
#define 	DRUN_MODE 	0x3

/*******************************************************************************
* External objects
*******************************************************************************/

/*******************************************************************************
* Global variables
*******************************************************************************/


/*******************************************************************************
* Local functions
*******************************************************************************/

/*******************************************************************************
Function Name : Sys_Init
Engineer      : b21190
Date          : Apr-04-2016
Parameters    : NONE
Modifies      : NONE
Returns       : NONE
Notes         : Clock settings
Issues        : NONE
*******************************************************************************/
void Sys_Init(void)
{
    //Enable external oscilator
    MC_ME.DRUN_MC.B.XOSCON = 1;

    // Set PLL0 to 200MHz
    PLLDIG.PLL0CR.B.CLKCFG = 1;		    //Bypass mode PLL0 on
    // RFDPHI1 = 10, RFDPHI = 2, PREDIV = 2, MFD = 14
    PLLDIG.PLL0DV.R = 0x50000000 |0x00020000 |0x00002000 |0x0014 ; //predefined PLL0 divider register

    // Set PPL0 as system clock
    MC_ME.DRUN_MC.B.PLL0ON = 1;             //Enable PLL0 for DRUN mode
    MC_ME.DRUN_MC.B.SYSCLK = 0x2;
    
    //  System clock dividers //
    
    // Enable system clock divider /4 -> 50MHz
    // (PBRIDGE_0, PBRIDGE_1, SIPI, DMA_CH_MUX) - Use only odd DIV values(2,4,etc..)
    //            Enable divider | divide by 4
    MC_CGM.SC_DC0.R = 0x80000000 | 0x30000;

    
    //  AUX_0 clock dividers //
    
    MC_CGM.AC0_SC.B.SELCTL =0x2;		//connect PLL0 to AXU_0
    
    // MOTC_CLK clock devider = 3 -> 200MHz/2 = 100MHz
    //             Enable divider | divide by 2
    MC_CGM.AC0_DC0.R = 0x80000000 | 0x10000;

    // SGEN_CLK clock devider = 10 -> 200MHz/10 = 20MHz
    //             Enable divider | divide by 10  
    MC_CGM.AC0_DC1.R = 0x80000000 | 0xA0000;

    // SAR ADC clock devider = 3 -> 200MHz/3 = 66.6MHz
    //             Enable divider | divide by 3  
    MC_CGM.AC0_DC2.R = 0x80000000 | 0x20000;

    
    //  AUX_1 clock dividers //
    
    // FRAY_CLK clock devider = 5 -> 200MHz/5 = 40MHz
    //             Enable divider | divide by 5  
    MC_CGM.AC1_DC0.R = 0x80000000 | 0x40000;

    // SENT_CLK clock devider = 3 -> 200MHz/3 = 66.6MHz
    //             Enable divider | divide by 3  
    MC_CGM.AC1_DC1.R = 0x80000000 | 0x20000;


    //  AUX_2 clock dividers //

    // CAN_CLK clock devider = 5 -> 200MHz/5 = 40MHz
    //             Enable divider | divide by 5
    MC_CGM.AC2_DC0.R = 0x80000000 | 0x40000;


    //  AUX_3 clock dividers //

    //	AUX Clock Selector 3 setup - source for PLL0 module
    MC_CGM.AC3_SC.B.SELCTL =1;			//connect (8..40MHz) XTALL to the PLL0 input
                           //0=IRC   1=XOSC

    
    //  AUX_4 clock dividers //
    
    //AUX Clock Selector 4 setup - source for PLL1 module
    MC_CGM.AC4_SC.B.SELCTL =3;			//connect PLL0 to AUX4

    
    //  AUX_5 clock dividers //

    MC_CGM.AC5_SC.B.SELCTL =0x2;		//connect PLL0 to AXU_5
    
    // LFAST PLL clock devider = 10 -> 200MHz/10 = 20MHz
    //            Enable divider | divide by 10  
    MC_CGM.AC5_DC0.R = 0x80000000 | 0x90000;


    //  AUX_6 clock dividers //

    MC_CGM.AC6_SC.R = 0x02000000;           	//connect PPL0 to AUX_6
    // CLKOUT_0 clock devider = 20 -> 200MHz/20 = 10MHz
    //            Enable divider | divide by 20  
    MC_CGM.AC6_DC0.R = 0x80000000 | 0x140000;

    
    //  AUX_10 clock dividers //
    
    MC_CGM.AC10_SC.R = 0x02000000;           	//connect PPL0 to AUX_10
    
    // ENET_CLK clock devider = 4 -> 200MHz/4 = 50MHz
    //            Enable divider | divide by 4  
    MC_CGM.AC10_DC0.R = 0x80000000 | 0x30000;

    
    //  AUX_11 clock dividers //
    
    MC_CGM.AC11_SC.R = 0x02000000;           	//connect PPL0 to AUX_11
    
    // ENET_TIME_CLK clock devider = 4 -> 200MHz/4 = 50MHz
    //            Enable divider | divide by 4  
    MC_CGM.AC11_DC0.R = 0x80000000 | 0x30000;

    //Mode transition to apply the PLL0 setup and set Normal mode with PLL running
    MC_ME.MCTL.R = 0x30005AF0;              //DRUN Mode & Key 
    MC_ME.MCTL.R = 0x3000A50F;              //DRUN Mode & Key

    while(!MC_ME.GS.B.S_PLL0);              //ME_GS Wait for PLL stabilization.
    while(MC_ME.GS.B.S_MTRANS);             //Waiting for end of transaction
    while(MC_ME.GS.B.S_CURRENT_MODE != DRUN_MODE);  // ME_GS Check DRUN mode has successfully been entered


    MC_ME.DRUN_MC.B.PLL1ON = 1;             //Enable PLL1 for DRUN mode

    //Configure RunPeripheralConfiguration registers in ME_RUN_PC0
    MC_ME.RUN_PC[0].B.DRUN = 1;             //Enable DRUN mode for all peripherals using ME_RUN_PC[0]
    // Set PLL1 to 200 MHz with 40MHz XOSC reference
    PLLDIG.PLL1DV.R = 0x00020014;	     // MFD = 20, RFDPHI = 2

    //Mode transition to apply the PLL0 setup and set Normal mode with PLL running
    MC_ME.MCTL.R = 0x30005AF0;              //DRUN Mode & Key 
    MC_ME.MCTL.R = 0x3000A50F;              //DRUN Mode & Key

    while(!MC_ME.GS.B.S_PLL0);              //ME_GS Wait for PLL stabilization.
    while(MC_ME.GS.B.S_MTRANS);             //Waiting for end of transaction
    while(MC_ME.GS.B.S_CURRENT_MODE != DRUN_MODE);  // ME_GS Check DRUN mode has successfully been entered
    
}//Sys_Init

/*******************************************************************************
Function Name : SUIL2_Init
Engineer      : b21190
Date          : Oct-22-2014
Parameters    : NONE
Modifies      : NONE
Returns       : NONE
Notes         : SIUL2 initialization (ports), setting PB6(B9) as output for clock measurement
Issues        : NONE
*******************************************************************************/
void SUIL2_Init(void)
{
   SIUL2.MSCR[22].R = 0x22800001;   	/* PB6 as CLK_OUT (on EVB it is B9)*/
}//SUIL2_Init

/*******************************************************************************
Function Name : SUIL2_Init
Engineer      : b21190
Date          : Oct-22-2014
Parameters    : NONE
Modifies      : NONE
Returns       : NONE
Notes         : SIUL2 initialization (ports), setting PB6(B9) as output for clock measurement
Issues        : NONE
*******************************************************************************/
void STOP_mode(void)
{
    //Mode transition to RUN0 mode with IRC running
    MC_ME.MCTL.R = 0x40005AF0;              //RUN0 Mode & Key 
    MC_ME.MCTL.R = 0x4000A50F;              //RUN0 Mode & Key

    //Enable STOP mode
    MC_ME.ME.B.STOP0 = 0x1;
    MC_ME.ME.B.HALT0 = 0x1;
    
    //Mode transition to STOP mode with IRC running
    MC_ME.MCTL.R = 0xA0005AF0;              //STOP Mode & Key 
    MC_ME.MCTL.R = 0xA000A50F;              //STOP Mode & Key
   
}//STOP_mode

/*******************************************************************************
* Global functions
*******************************************************************************/
void main (void)
{
  //Sys_Init();
  SUIL2_Init();
  STOP_mode();

  while(1);
}//main
